<?php
/*
 * Plugin Name: HTTPS Debugger
 * Version: 1.1
 * Description: Used to determine if there's anything wrong with the communication to WordPress.org over HTTPS. See Dashboard -> HTTPS Tester (If using multisite, visit Updates -> HTTPS Tester in the network admin.)
 * Plugin URI: http://wordpress.org/
 * Author: the WordPress team
 * Author URI: http://wordpress.org/
 * Version: 1.1
 * Network: true
 */

class HTTPS_Tester {

	function __construct() {
		if ( is_multisite() )
			add_action( 'network_admin_menu', array( $this, 'admin_menu' ) );
		else
			add_action( 'admin_menu', array( $this, 'admin_menu' ) );
	}

	function admin_menu() {
		$top_level_menu = is_multisite() ? 'update-core.php' : 'index.php';
		add_submenu_page(
			$top_level_menu,
			'HTTPS Tester',
			'HTTPS Tester',
			'manage_options',
			'https-tester',
			array( $this, 'content_wrapper' )
		);
	}

	function content_wrapper() {
		include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>HTTPS Tester</h2>';
		$this->content();
		echo '</div>';
	}

	function content() {
		echo '<p>Since WordPress 3.7, all communication to WordPress.org is attempted over HTTPS, this is to improve security and make it harder for someone to perform a MITM attack against a WordPress site.</p>';
		echo "<p>Unfortunately, there have been reports that some hosts configurations are not allowing it to work, this plugin is used to debug it and find out what's going on.</p>";

		echo '<ol>';
		foreach ( $this->tests() as $item ) {

			switch ( $item->severity ) {
				case 'warning':
					$status = '<span style="color:orange;">' . __( 'WARNING' ) . '</span>';
					break;
				case 'pass':
					$status = '<span style="color:green;">' . __( 'PASS' ) . '</span>';
					break;
				case 'info':
					$status = '<span style="color:darkgreen;">' . __( 'INFO' ) . '</span>';
					break;
				case 'fail':
					$status = '<span style="color:red;">' . __( 'FAIL' ) . '</span>';
					break;
			}

			printf( '<li>[%s]: %s', $status, $item->desc );
		}
		echo '</ol>';
	}

	function tests() {
		$tests = array();
		foreach ( get_class_methods( $this ) as $method ) {
			if ( 'test_' != substr( $method, 0, 5 ) )
				continue;

			$result = $this->$method();

			// Is it applicable to this install
			if ( false === $result )
				continue;

			$result = (object) $result;
			if ( empty( $result->severity ) )
				$result->severity = 'warning';

			$tests[ $method ] = $result;
		}
		return $tests;
	}

	function test_https_supported() {
		$support = wp_http_supports( array( 'ssl' ) );
		return array(
			'desc' => $support ?
				'Your WordPress install claims to support HTTPS Connections' :
				"Your WordPress install doesn't appear to support HTTPS communication, that's OK, Communication with WordPress.org will continue over HTTP, but you'll not get Automatic Updates. You should ask your host to install the <code>openssl</code> PHP extension, and/or to install <code>cURL</code> with HTTPS support",
			'severity' => $support ? 'pass' : 'fail',
		);
	}

	function test_certificate_file_exists() {
		return array(
			'desc' => "Checking that the HTTPS Root Certificate bundle exists and is accessible",
			'severity' => is_readable( ABSPATH . WPINC . '/certificates/ca-bundle.crt' ) ? 'pass' : 'fail',
		);
	}

	function test_cURL_installed() {
		if ( ! extension_loaded( 'curl' ) ) {
			return array(
				'desc' => 'cURL is not installed on this server',
				'severity' => 'warning'
			);
		}
		$curl_version = curl_version();
		$curl_supports_https = (CURL_VERSION_SSL & $curl_version['features']);
		$details = '';
		foreach ( $curl_version as $k => $v ) {
			$details .= "; $k=" . implode( ',', (array) $v );
		$details = trim( $details, '; ' );
		}
		if ( ! $curl_supports_https ) {
			return array(
				'desc' => sprintf( "cURL is installed but doesn't support SSL communication, cURL Details: %s", $details ), 
				'severity' => 'warning',
			);
		}
		return array(
			'desc' => sprintf( "cURL is installed and supports SSL communication, cURL Details: %s", $details ),
			'severity' => 'pass'
		);
	}

	function test_streams_available() {
		if ( ! extension_loaded( 'openssl' ) ) {
			return array(
				'desc' => 'OpenSSL is not installed',
				'severity' => 'fail'
			);
		}
		return array(
			'desc' => sprintf( 'OpenSSL is installed. %s %s', OPENSSL_VERSION_TEXT, OPENSSL_VERSION_NUMBER ),
			'severity' => 'pass'
		);
	}

	function test_stream_socket_client() {
		return array(
			'desc' => 'Checking if stream_socket_client exists',
			'severity' => function_exists( 'stream_socket_client' ) ? 'pass' : 'fail'
		);
	}

	function test_parse_x509() {
		return array(
			'desc' => 'Checking if openssl_x509_parse exists',
			'severity' => function_exists( 'openssl_x509_parse' ) ? 'pass' : 'fail'
		);
	}

	function test_api_wordpress_org_ips() {
		$ip = gethostbyname( 'api.wordpress.org' );
		$expected = array( '66.155.40.250', '66.155.40.249' );
		$ok = in_array( $ip, $expected );
		return array(
			'desc' => sprintf( 'Verifying api.wordpress.org resolves correctly. %s', $ok ? '' : $ip ),
			'severity' => $ok ? 'pass' : 'fail'
		);
	}

	function _test_api_wordpress_org_communication( $with_body = false ) {
		$expected_content = '4b29b5d408e24d2dd1f2dda7d3570b11';

		$timer = microtime( true );
		$opts = array(
			'timeout' => 20,
		);
		if ( $with_body ) {
			$opts['body'] = array(
				'foo' => 'bar',
			);
		}
		$result = wp_remote_post( 'https://api.wordpress.org/', $opts );
		$time_taken = round( microtime( true ) - $timer, 3 );

		if ( is_wp_error( $result ) ) {
			return array(
				'desc' => sprintf( "Communication with WordPress.org failed with error: [%s]: %s", $result->get_error_code(), $result->get_error_message() ),
				'severity' => 'fail',
			);
		}

		if ( md5( $result['body'] ) != $expected_content ) {
			return array(
				'desc' => sprintf( 'Communication with WordPress.org suceeded after %s seconds, but an unexpected response was received: %s', $time_taken, esc_html( $result['body'] ) ),
				'secerity' => 'warning',
			);
		}

		return array(
			'desc' => sprintf( 'Communication with WordPress.org suceeded, it took %s seconds', $time_taken ),
			'severity' => 'pass',
		);
		
		return false;
	}

	function test_api_wordpress_org_communication_streams() {
		if ( ! WP_Http_Streams::test( array( 'ssl' => true ) ) )
			return false;

		add_filter( 'use_curl_transport', '__return_false' );
		$result = $this->_test_api_wordpress_org_communication();
		remove_filter( 'use_curl_transport', '__return_false' );		

		$result['desc'] = '[Streams] ' . $result['desc'];
		return $result;
	}

	function test_api_wordpress_org_communication_streams_post_body() {
		if ( ! WP_Http_Streams::test( array( 'ssl' => true ) ) )
			return false;

		add_filter( 'use_curl_transport', '__return_false' );
		$result = $this->_test_api_wordpress_org_communication( true );
		remove_filter( 'use_curl_transport', '__return_false' );		

		$result['desc'] = '[Streams with a POST body] ' . $result['desc'];
		return $result;
	}

	function test_api_wordpress_org_communication_curl() {
		if ( ! WP_Http_Curl::test( array( 'ssl' => true ) ) )
			return false;

		add_filter( 'use_streams_transport', '__return_false' );
		$result = $this->_test_api_wordpress_org_communication();
		remove_filter( 'use_streams_transport', '__return_false' );		

		$result['desc'] = '[cURL] ' . $result['desc'];
		return $result;
	}

	function test_api_wordpress_org_communication_curl_post_body() {
		if ( ! WP_Http_Curl::test( array( 'ssl' => true ) ) )
			return false;

		add_filter( 'use_streams_transport', '__return_false' );
		$result = $this->_test_api_wordpress_org_communication( true );
		remove_filter( 'use_streams_transport', '__return_false' );		

		$result['desc'] = '[cURL with a POST body] ' . $result['desc'];
		return $result;
	}

	function test_phpinfo() {
		return array(
			'desc' => sprintf( "PHP Version: %s", phpversion() ),
			'severity' => 'info'
		);
	}

}
new HTTPS_Tester();